
// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "FDExtCaptureAsync.h"
#include "MainFrm.h"
#include "SelectDlg.h"
#include <vector>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	ON_WM_CREATE()
	ON_WM_SETFOCUS()
	ON_WM_DESTROY()
	ON_WM_TIMER()
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

// CMainFrame construction/destruction

CMainFrame::CMainFrame() noexcept
{
	// TODO: add member initialization code here
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	// create a view to occupy the client area of the frame
	if (!m_wndView.Create(nullptr, nullptr, AFX_WS_DEFAULT_VIEW, CRect(0, 0, 0, 0), this, AFX_IDW_PANE_FIRST, nullptr))
	{
		TRACE0("Failed to create view window\n");
		return -1;
	}

	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP | CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}
	m_wndStatusBar.SetIndicators(indicators, sizeof(indicators)/sizeof(UINT));

	// TODO: Delete these three lines if you don't want the toolbar to be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	if (!InitAll()) {
		return -1;
	}
	SetTimer(101, 10, NULL);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	cs.style = WS_OVERLAPPED | WS_CAPTION | FWS_ADDTOTITLE
		 | WS_THICKFRAME | WS_MINIMIZEBOX | WS_MAXIMIZEBOX;

	cs.dwExStyle &= ~WS_EX_CLIENTEDGE;
	cs.lpszClass = AfxRegisterWndClass(0);
	return TRUE;
}

// CMainFrame diagnostics
#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}
#endif //_DEBUG


// CMainFrame message handlers

void CMainFrame::OnSetFocus(CWnd* /*pOldWnd*/)
{
	// forward focus to the view window
	m_wndView.SetFocus();
}

BOOL CMainFrame::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
	// let the view have first crack at the command
	if (m_wndView.OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
		return TRUE;

	// otherwise, do default handling
	return CFrameWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

void CMainFrame::OnDestroy()
{
	KillTimer(101);
	CFrameWnd::OnDestroy();
	ReleaseAll();
}

void CMainFrame::OnTimer(UINT_PTR nIDEvent)
{
	while (DoNext()) {}
	CFrameWnd::OnTimer(nIDEvent);
}


// FDExt SDK ------------------------------------------------------------------------------------
//
//
bool CMainFrame::InitAll()
{
	// create FDExt board's manager
	CComPtr<FDExtSrvdLib::IFDExtMgr> pIMgr;
	HRESULT hr = pIMgr.CoCreateInstance(FDExtSrvdLib::CLSID_FDExtMgr);
	if (NULL == pIMgr) {
		MessageBox(_T("No FDExt software!"));
		return false;
	}
	// scan all capture devices
	CSelectDlg dlg(this);
	std::vector<long> indexes;
	long nObjIndex = 0;
	long nCaptureCnt = 0;
	FDExtSrvdLib::CFDObjectInfo info;
	while (S_OK == (hr = pIMgr->get_NextValidFDObjectInfo(&nObjIndex, FDExtSrvdLib::CFDObjType::FDObjType_FreeRunVideoSrc, &info))) {
		CStringW szName;
		szName.Format(L"%d: %s / %s", ++nCaptureCnt, (LPCWSTR)info.m_BoardName, (LPCWSTR)info.m_FDObjName);
		dlg.AddName(szName);
		indexes.push_back(nObjIndex - 1);
	}
	if (0 == nCaptureCnt) {
		MessageBox(_T("No FDExt capture source!"));
		return false;
	}
	// ask user to select the device
	dlg.DoModal();
	nObjIndex = dlg.GetSelectedIndex();
	if (nObjIndex < 0) {
		return false;
	}
	// create selected capture object (by index)
	CComPtr<IUnknown> pIUnk;
	hr = pIMgr->get_FDObjectByIndex(indexes[nObjIndex], FDExtSrvdLib::CFDObjType::FDObjType_FreeRunVideoSrc, &pIUnk);
	if (NULL == pIUnk) {
		MessageBox(_T("Internal error #1"));
		return false;
	}
	hr = pIUnk->QueryInterface(&m_pICapture);
	if (NULL == m_pICapture) {
		MessageBox(_T("Internal error #2"));
		return false;
	}
	// remeber start frame number
	m_pICapture->FDGetVideoTime(&m_nLastFrameNumber, &m_nLastFrameTime);
	m_nLastFrameTick = ::GetTickCount();
	return true;
}
//
void CMainFrame::ReleaseAll()
{
	m_pICapture = NULL;
}
//
bool CMainFrame::DoNext()
{
	if (NULL == m_pICapture)
		return false;
	// check newerst frame number exist
	ULONGLONG frame, time;
	HRESULT hr = m_pICapture->FDGetVideoTime(&frame, &time);
	ATLASSERT(S_OK == hr);
	if (S_OK != hr)
		return false;
	if (frame == m_nLastFrameNumber)
		return false;
	if (frame > m_nLastFrameNumber + 5) {
		// drop many frames => reset to new frame position
		m_nLastFrameNumber = frame - 1;
	}
	// try to obtain new frame info
	frame = m_nLastFrameNumber + 1;
	FDExtSrvdLib::tag_CFrameInfo frameInfo;
	hr = m_pICapture->GetFrameInfo(frame, &frameInfo);
	if (S_OK != hr)
		return false;
	ULONGLONG duration = (time - m_nLastFrameTime + 4999) / 10000;
	// capture image
	if (frameInfo.m_IsProgressive) // full frame
		m_wndView.SetUYVY(frameInfo.m_pVideo[0], frameInfo.m_VideoStride, frameInfo.m_FrameDX, frameInfo.m_FrameDY);
	else // upper field only
		m_wndView.SetUYVY(frameInfo.m_pVideo[0], frameInfo.m_VideoStride, frameInfo.m_FrameDX, frameInfo.m_FrameDY/2);
	// all done - update status info
	m_nLastFrameNumber = frame;
	m_nLastFrameTime = time;
	CString str;  str.Format(_T("Frame %lld: %dx%d dur=%lldms %s fmt=%d."), m_nLastFrameNumber, frameInfo.m_FrameDX, frameInfo.m_FrameDY, duration, frameInfo.m_IsProgressive ? _T("Progressive") : _T("Interlaced"), frameInfo.m_DataFormat);
	m_wndStatusBar.SetWindowText(str);
	return true;
}
